<?php
if (!class_exists('SettingsLib')) {
class SettingsLib {
	var $settings;
	var $name;
	var $group;
	var $main;
	var $permission;
	var $icon;
	var $position;
	var $html;

	function __construct($settings, $name = 'Settings', $group = '', $main = false, $permission = 'manage_options', $icon = null, $position = null, $html = '') {
		$this->settings = $settings ? $settings : array();
		$this->name = $name;
		$this->group = $group;
		$this->main = $main;
		$this->permission = $permission;
		$this->icon = $icon;
		$this->position = $position;
		$this->html = $html;
		add_action('admin_init', array($this, 'admin_init'));
		add_action('admin_menu', array($this, 'admin_menu'));
	}
	public function admin_init() {
		foreach ($this->settings as $setting) {
			$setting = (object) $setting;
			$type = $setting->type;
			$type = $type == 'select' ? 'string' : $type;
			$type = $type == 'text' ? 'string' : $type;
			$type = $type == 'url' ? 'string' : $type;
			$type = $type == 'password' ? 'string' : $type;
			register_setting($this->group, $setting->id,
				array(
					'type' => $type,
					'description' => $setting->title,
					'default' => property_exists($setting, 'default') ? $setting->default : null
				)
			);
		}
	}
	public function admin_menu() {
		if ($this->main) {
			add_menu_page(
				$this->name,
				$this->name,
				$this->permission,
				self::create_key($this->group),
				array($this, 'menu_page'),
				$this->icon,
				$this->position
			);
		} else {
			add_submenu_page(
				self::create_key($this->group),
				$this->name,
				$this->name,
				$this->permission,
				self::create_key($this->group.' '.$this->name),
				array($this, 'menu_page')
			);
		}
	}
	public static function create_key($key) {
		return urlencode(str_replace(' ','-',strtolower($key)));
	}
	public static function notice($message, $type = 'success') {
		echo '<div class="notice notice-'.$type.' is-dismissible"> 
			<p><strong>'.$message.'</strong></p>
		</div>';
	}
	public static function check_nounce($key) {
		if (isset($_POST[$key])) {
			if (wp_verify_nonce($_POST[$key], plugin_basename( __FILE__ ))) {
				return true;
			} else {
				self::notice('Security check failed, please login and try again.');
			}
		}
		return false;
	}
	public static function echo_nounce($key) {
		wp_nonce_field(plugin_basename( __FILE__ ), $key);
	}
	public function settings($setting) {
		//TODO: add select
		$setting = (object) $setting;
		if ($setting->type == 'boolean') {
			echo $this->settings_checkbox($setting);
		} elseif ($setting->type == 'select') {
			echo $this->settings_select($setting);
		} elseif ($setting->type == 'text') {
			echo $this->settings_text($setting);
		} else {
			echo $this->settings_input($setting);
		}
	}
	public function settings_row($setting, $html) {
		return '
		<tr valign="top">
			<th scope="row">'.$setting->title.($setting->type=='text'?'<div style="font-weight:normal;">'.$setting->description.'</div>':'').'</th>
			<td>'.$html.'</td>
		</tr>';
	}
	public function settings_text($setting) {
		$html = '<textarea id="'.$setting->id.'" name="'.$setting->id.'" rows="10" cols="70" />'.esc_html(get_option($setting->id)).'</textarea>';
		return $this->settings_row($setting, $html);
	}
	public function settings_select($setting) {
		$assoc = self::associative($setting->values);
		$current = get_option($setting->id);
		$html = '<select id="'.$setting->id.'" name="'.$setting->id.'">';
		foreach ($setting->values as $key => $name) {
			$value = $assoc ? $key : $name;
			$html .= '<option value="'.esc_attr($value).'"'.($current==$value?' selected="selected"':'').'>'.esc_html($name).'</option>';
		}
		$html .= '</select>';
		return $this->settings_row($setting, $html);
	}
	public function settings_checkbox($setting) {
		$html = '<input type="checkbox" id="'.$setting->id.'" name="'.$setting->id.'"'.(get_option($setting->id) == '1' ? ' checked="checked"' : '').' />';
		$html .= '<label for="name="'.$setting->id.'"">'.($setting->description?$setting->description:'Enable').'</label>';
		return $this->settings_row($setting, $html);
	}
	public function settings_input($setting) {
		$html = '<input style="width:520px;" placeholder="'.$setting->description.'" type="'.($setting->type=='password'?'password':'text').'" name="'.$setting->id.'" value="'.esc_attr(get_option($setting->id)).'" />';
		//$html .= '<div>'.$setting->description.'</div>';
		return $this->settings_row($setting, $html);
	}
	public function checkbox_request($key) {
		return isset($_REQUEST[$key]) && $_REQUEST[$key] != '' ? 1 : 0;
	}
	public function process_checkbox($key) {
		update_option($key, $this->checkbox_request($key));
	}
	public function process_option($key) {
		update_option($key, sanitize_text_field($_REQUEST[$key]));
	}
	public function associative($array) {
		return array_keys($array) !== range(0, count($array) - 1);
	}
	public function menu_page() {
		echo '<h1>'.$this->name.'</h1>';
		echo $this->html;
		$nounce = $this->group;
		if (self::check_nounce($nounce)) {
			foreach ($this->settings as $setting) {
				$setting = (object) $setting;
				if ($setting->type == 'boolean') {
					$this->process_checkbox($setting->id);
				} else {
					$this->process_option($setting->id);
				}
			}
			self::notice('Saved.');
		}
		?>
		<form method="post">
		<?php self::echo_nounce($nounce); ?>
		<table class="form-table">
			<?php
			foreach ($this->settings as $setting) {
				$this->settings((object) $setting);
			}
			?>
		</table>
		<?php submit_button(); ?>
		</form>
		<?php
	}
}
}